Package: desplot
Title: Plotting Field Plans for Agricultural Experiments
Version: 1.10
Authors@R: person("Kevin","Wright", email="kw.stat@gmail.com", comment=c(ORCID = "0000-0002-0617-8673"), role=c("aut","cre"))
Type: Package
Description: A function for plotting maps of agricultural field experiments that are laid out in grids.  See Ryder (1981) <doi:10.1017/S0014479700011601>.
Imports: ggplot2, grid, lattice, reshape2, rlang
Suggests: agridat, knitr, rmarkdown, testthat
License: GPL-3
URL: https://kwstat.github.io/desplot/
BugReports: https://github.com/kwstat/desplot/issues
VignetteBuilder: knitr
RoxygenNote: 7.2.2
Encoding: UTF-8
Language: en-US
NeedsCompilation: no
Packaged: 2023-03-09 22:45:26 UTC; wrightkevi
Author: Kevin Wright [aut, cre] (<https://orcid.org/0000-0002-0617-8673>)
Maintainer: Kevin Wright <kw.stat@gmail.com>
Repository: CRAN
Date/Publication: 2023-03-09 23:20:06 UTC
Built: R 4.2.0; ; 2023-07-11 01:15:47 UTC; unix
