# Generated by roxygen2: do not edit by hand

export(RedGrayBlue)
export(StatTileBorder)
export(desplot)
export(geom_tileborder)
export(ggdesplot)
export(panel.outlinelevelplot)
import(ggplot2)
import(grid)
import(lattice)
importFrom(ggplot2,Stat)
importFrom(ggplot2,ggproto)
importFrom(reshape2,acast)
importFrom(reshape2,melt)
importFrom(rlang,.data)
importFrom(stats,as.formula)
importFrom(stats,formula)
importFrom(stats,median)
importFrom(stats,update)
